@@ -2,8 +2,8 @@
 
 # Author: Iago Mosqueira, AZTI Tecnalia
 # Additions:
-# Last Change: 28 feb 2006 17:41
-# $Id: FLAccesors.R,v 1.1.2.3 2006/03/13 12:12:14 iagoazti Exp $
+# Last Change: 15 nov 2006 20:59
+# $Id: FLAccesors.R,v 1.1.2.4 2007/01/16 17:28:48 imosqueira Exp $
 
 # Reference:
 # Notes:
@@ -20,13 +20,14 @@ createFLAccesors <- function(object, exclude=character(1)) {
 	for (x in names(slots)) {
 		# check method is defined already and signatures match
 		eval(
-		substitute(if(isGeneric(x) && names(formals(x)) != "object") {warning(paste("Accesor method for",
-			x, "conflicts with a differently defined generic. Type", x,  "for more information")); break},
-			list(x=x))
+		substitute(if(isGeneric(x) && names(formals(x)) != "object") {warning(paste("Accesor
+			method for", x, "conflicts with a differently defined generic. Type", x,
+			"for more information")); break}, list(x=x))
 			)
 		# create new generic and accesor method
 		eval(
-		substitute(if(!isGeneric(x)) setGeneric(x, function(object, ...) standardGeneric(x)), list(x=x))
+		substitute(if(!isGeneric(x)) setGeneric(x, function(object, ...) standardGeneric(x)),
+		list(x=x))
 		)
 		eval(
 		substitute(setMethod(x, signature(y), function(object) return(slot(object, x))), list(x=x,
@@ -40,7 +41,8 @@ createFLAccesors <- function(object, exclude=character(1)) {
 		)
 		eval(
 		substitute(setMethod(x, signature(object=y, value=v), function(object, value)
-			{slot(object, s) <- value; object}), list(x=xr, y=class(object), s=x, v=unname(slots[x])))
+			{slot(object, s) <- value; object}), list(x=xr, y=class(object), s=x,
+			v=unname(slots[x])))
 		)
 		defined[[x]] <- c(x, xr, paste('alias{',x,',',class(object),'-method}', sep=''),
 			paste('\alias{',xr,',',class(object),',',unname(slots[x]), '-method}', sep=''),
