@@ -1,4 +1,4 @@
 .onLoad <- function(lib,pkg) {
 	require(methods)
-	cat("FLCore 1.3-3 - \"The Dutch Hamster Redux\" \n")
+	cat("FLCore 1.4-3 - \"Golden Jackal\" \n")
 }
